import pyglet
import pygame
import os

from settings import Settings

from colors import *

if os.name == 'nt':
    pygame.mixer.pre_init(44100, -16, 2, 2048)
pygame.mixer.init()
pygame.mixer.set_num_channels(36)
Sound = pygame.mixer.Sound
Channel = pygame.mixer.Channel

batch = pyglet.graphics.Batch()

volume = Settings.get_option('volume')

class String(object):
    def __init__(self, x1, y1, x2, y2):
        self.x = x1
        self.y = y1
        self.base_color = [0, 0, 0, 255] * 2
        self.dim_color = [0, 0, 0, 75] * 2
        self.played_color = [200, 200, 200, 255] * 2
        self.select_color = [153, 51, 51, 255] * 2
        self.current_color = self.base_color
        self._list = batch.add(2, pyglet.gl.GL_LINES, None,
                ('v2f', [x1, y1, x2, y2]),
                ('c4B', [0, 0, 0, 255] * 2))
        self.muted = False

    def reset(self):
        self._list.colors = self.base_color
        self.current_color = self.base_color

    def select(self):
        self.current_color = self.select_color
        if not self.muted:
            self._list.colors = self.select_color

    def unselect(self):
        self.current_color = self.base_color
        if not self.muted:
            self._list.colors = self.base_color
        else:
            self._list.colors = self.dim_color

    def mute(self):
        self._list.colors = self.dim_color
        self.muted = True

    def unmute(self):
        self._list.colors = self.current_color
        self.muted = False

    def activate(self):
        self._list.colors = self.played_color

    def deactivate(self):
        self._list.colors = self.current_color

    def play(self):
        self.channel.set_volume(volume / 10.)
        self.channel.play(self.sound)

    def stop(self):
        self.channel.stop()

def get_path(bank, name):
    return os.path.join('data', bank, name)

def sound_bank(bank):
    return [Sound(get_path(bank, 'G2.ogg')),   #1
           Sound(get_path(bank, 'A2.ogg')),
           Sound(get_path(bank, 'C3.ogg')),
           Sound(get_path(bank, 'D3.ogg')),
           Sound(get_path(bank, 'E3.ogg')),
           Sound(get_path(bank, 'F#3.ogg')),  #6
           Sound(get_path(bank, 'G3.ogg')),
           Sound(get_path(bank, 'A3.ogg')),
           Sound(get_path(bank, 'B3.ogg')),
           Sound(get_path(bank, 'C4.ogg')),
           Sound(get_path(bank, 'C4.ogg')),   #11
           Sound(get_path(bank, 'C#4.ogg')),
           Sound(get_path(bank, 'D4.ogg')),
           Sound(get_path(bank, 'D4.ogg')),
           Sound(get_path(bank, 'E4.ogg')),
           Sound(get_path(bank, 'E4.ogg')),   #16
           Sound(get_path(bank, 'F#4.ogg')),
           Sound(get_path(bank, 'G4.ogg')),
           Sound(get_path(bank, 'G4.ogg')),
           Sound(get_path(bank, 'A4.ogg')),
           Sound(get_path(bank, 'A4.ogg')),   #21
           Sound(get_path(bank, 'B4.ogg')),
           Sound(get_path(bank, 'C5.ogg')),
           Sound(get_path(bank, 'C5.ogg')),
           Sound(get_path(bank, 'C#5.ogg')),
           Sound(get_path(bank, 'D5.ogg')),   #26
           Sound(get_path(bank, 'D5.ogg')),
           Sound(get_path(bank, 'E5.ogg')),
           Sound(get_path(bank, 'E5.ogg')),
           Sound(get_path(bank, 'F#5.ogg')),
           Sound(get_path(bank, 'G5.ogg')),   #31
           Sound(get_path(bank, 'G5.ogg')),
           Sound(get_path(bank, 'A5.ogg')),
           Sound(get_path(bank, 'A5.ogg')),
           Sound(get_path(bank, 'B5.ogg')),
           Sound(get_path(bank, 'C6.ogg'))]   #36

sounds = sound_bank(Settings.get_option('sound'))

strings = [String(85, 68, 723, 68),
           String(100, 76, 722, 76),
           String(85, 85, 721, 85),
           String(100, 93, 720, 93),
           String(85, 102, 719, 102),
           String(100, 111, 718, 111),
           String(85, 119, 717, 119),
           String(100, 127, 716, 127),
           String(85, 136, 715, 136),
           String(100, 145, 714, 145),
           String(85, 153, 713, 153),
           String(100, 161, 712, 161),
           String(85, 170, 711, 170),
           String(96, 178, 710, 178),
           String(110, 186, 709, 186),
           String(124, 194, 708, 194),
           String(138, 202, 707, 202),
           String(152, 210, 706, 210),
           String(166, 218, 705, 218),
           String(180, 226, 704, 226),
           String(194, 234, 703, 234),
           String(208, 242, 702, 242),
           String(222, 250, 701, 250),
           String(236, 258, 700, 258),
           String(250, 266, 699, 266),
           String(264, 274, 698, 274),
           String(278, 282, 697, 282),
           String(292, 290, 696, 290),
           String(306, 298, 695, 298),
           String(320, 306, 694, 306),
           String(334, 314, 693, 314),
           String(348, 322, 692, 322),
           String(362, 330, 691, 330),
           String(376, 338, 690, 338),
           String(390, 346, 689, 346),
           String(404, 354, 688, 354)]

def set_sounds():
    for string, n in zip(strings, range(36)):
        channel = Channel(n)
        channel.set_volume(volume / 10.)
        string.channel = channel
        string.sound = sounds[n]

set_sounds()

# not playable strings
String(85, 68, 36, 92)
String(100, 76, 53, 88)
String(85, 85, 36, 109)
String(100, 93, 53, 105)
String(85, 102, 36, 126)
String(100, 111, 53, 122)
String(85, 119, 36, 143)
String(100, 127, 53, 139)
String(85, 136, 36, 160)
String(100, 145, 53, 156)
String(85, 153, 36, 177)
String(100, 161, 53, 173)
String(85, 170, 47, 186)
String(96, 178, 70, 189)
String(110, 186, 70, 207)
String(124, 194, 96, 207)
String(138, 202, 96, 225)
String(152, 210, 124, 223)
String(166, 218, 124, 241)
String(180, 226, 152, 239)
String(194, 234, 152, 257)
String(208, 242, 180, 255)
String(222, 250, 180, 273)
String(236, 258, 208, 271)
String(250, 266, 208, 289)
String(264, 274, 236, 287)
String(278, 282, 236, 305)
String(292, 290, 264, 303)
String(306, 298, 264, 321)
String(320, 306, 292, 319)
String(334, 314, 292, 337)
String(348, 322, 320, 335)
String(362, 330, 320, 353)
String(376, 338, 348, 351)
String(390, 346, 348, 369)
String(404, 354, 376, 367)
