import math

import pyglet
pyglet.options['debug'] = False
from pyglet.gl import *
pyglet.resource.path.append('data')
pyglet.resource.reindex()
pyglet.resource.add_font('title.ttf')
pyglet.resource.add_font('ProggySquare.ttf')

import button
from colors import *
import gstring
import modes
from settings import Settings

class Window(pyglet.window.Window):
    def __init__(self):
        super(Window, self).__init__(800, 600, caption='Plectrum Infinitum')
        self.setup_gl()

        self.background = pyglet.resource.image('body_' + Settings.get_option('style') + '.png')
        self.fretboard = pyglet.resource.image('fretboard.png')
        self.hardware = pyglet.resource.image('hardware.png')

        self.sounds = gstring.sounds
        self.mode = None
        self.help_mode()

        self.pause = False

        pyglet.clock.schedule(self.update)
        
    def setup_gl(self):
        pyglet.gl.glClearColor(*Black.normala())
        pyglet.gl.glEnable(pyglet.gl.GL_LINE_SMOOTH)
        pyglet.gl.glEnable(pyglet.gl.GL_BLEND)
        pyglet.gl.glBlendFunc(pyglet.gl.GL_SRC_ALPHA,
                pyglet.gl.GL_ONE_MINUS_SRC_ALPHA)

    def _set_mode(self, mode):
        self.remove_handlers(self.mode)
        self.mode = mode
        self.push_handlers(self.mode)

    def menu_mode(self):
        self.set_mouse_cursor(pyglet.window.ImageMouseCursor(pyglet.resource.image('pick.png'), 5, 2))
        self._set_mode(modes.MenuMode(self))

    def help_mode(self):
        self._set_mode(modes.HelpMode(self))

    def play_mode(self, level):
        self._set_mode(modes.PlayMode(self, level))

    def jam_mode(self):
        self._set_mode(modes.JamMode(self))

    def edit_mode(self):
        self._set_mode(modes.EditMode(self))

    def outro_mode(self):
        self._set_mode(modes.OutroMode(self))

    def select_mode(self):
        self._set_mode(modes.SelectMode(self))

    def update(self, dt):
        if self.pause:
            return
        self.mode.update(dt)

    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.P:
            self.pause = not self.pause

    def on_draw(self):
        self.clear()
        # FIXME: There is obviously either a hardware or pyglet issue going on.
        try:
            self.background.blit(0,0)
        except:
            pass
        gstring.batch.draw()
        self.hardware.blit(0,0)
        self.fretboard.blit(0,0)
        self.mode.draw()

def main():
    window = Window()
    pyglet.app.run()
