import pyglet
import gstring
import button

from colors import *
from settings import Settings

class SlideSelect(object):
    def __init__(self, y, label, items, current, callback, image):
        self.items = items
        self.callback = callback
        self.current = current
        self.label = pyglet.text.Label(label, font_size=24, x=560, y=y, color=Grey.rgba(), anchor_x='center')
        self.left_arrow = pyglet.sprite.Sprite(image, x=470, y=y+2)
        self.right_arrow = pyglet.sprite.Sprite(image, x=640, y=y+2)

    def draw(self):
        self.label.draw()
        self.left_arrow.draw()
        self.right_arrow.draw()

    def on_mouse_press(self, x, y, buttons, modifiers):
        for arrow, num in zip([self.left_arrow, self.right_arrow], [0, 1]):
            if x >= arrow.x and x <=  arrow.x + arrow.width and\
                y >= arrow.y and y <= arrow.y + arrow.height:
                if num == 0:
                    self.current -= 1
                    if self.current < 0:
                        self.current = len(self.items) - 1
                elif num == 1:
                    self.current += 1
                    if self.current >= len(self.items):
                        self.current = 0
                self.callback(self.items[self.current])

class EditMode(object):
    def __init__(self, window):
        self.window = window
        self.title = pyglet.resource.image('title.png')
        styles = ['body_wood.png', 'body_metal.png']
        style_start = styles.index('body_' + Settings.get_option('style') + '.png')
        sounds = ['dulcimer', 'piano']
        sound_start = sounds.index(Settings.get_option('sound'))
        self.selectors = [SlideSelect(245, 'style', styles, style_start, self.change_style, pyglet.resource.image('blue_note.png')),
                          SlideSelect(195, 'sound', sounds, sound_start, self.change_sound, pyglet.resource.image('green_note.png')),
                          SlideSelect(145, 'volume', range(11), gstring.volume, self.change_volume, pyglet.resource.image('orange_note.png'))]

    def change_style(self, name):
        self.window.background = pyglet.resource.image(name)
        Settings.set_option('style', name[5:-4])

    def change_sound(self, name):
        gstring.sounds = gstring.sound_bank(name)
        gstring.set_sounds()
        Settings.set_option('sound', name)
        for n in range(36):
            if n + 1 not in button.DButton.chord:
                gstring.strings[n].play()

    def change_volume(self, number):
        gstring.volume = number
        Settings.set_option('volume', number)
        for n in range(36):
            if n + 1 not in button.SButton.chord:
                gstring.strings[n].play()

    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.ESCAPE:
            Settings.save()
            self.window.menu_mode()

    def on_mouse_press(self, x, y, buttons, modifiers):
        for selector in self.selectors:
            selector.on_mouse_press(x, y, buttons, modifiers)

    def update(self, dt):
        pass

    def draw(self):
        self.title.blit(0, 0)
        for selector in self.selectors:
            selector.draw()
