import pyglet

import button
import gstring
from plectrum import Plectrum

class JamMode(object):
    def __init__(self, window):
        self.window = window
        self.muted = []
        self.plectrum = Plectrum()
        self.strumming = False

    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.ESCAPE:
            self.window.menu_mode()
        if symbol == pyglet.window.key.SPACE:
            for string in gstring.strings:
                string.channel.stop()
        try:
            button.Buttons[symbol].on = True
            for note in button.Buttons[symbol].chord:
                self.muted.append(note)
                gstring.strings[note - 1].stop()
            for n in self.muted:
                gstring.strings[n - 1].mute()
        except KeyError:
            pass

    def on_key_release(self, symbol, modifiers):
        try:
            button.Buttons[symbol].on = False
            for note in button.Buttons[symbol].chord:
                self.muted.remove(note)
            for n in range(36):
                if n + 1 not in self.muted:
                    gstring.strings[n].unmute()
        except KeyError:
            pass

    def on_mouse_press(self, x, y, buttons, modifiers):
        self.strumming = True

    def on_mouse_release(self, x, y, buttons, modifiers):
        self.strumming = False

    def on_mouse_drag(self, x, y, dx, dy, button, modifiers):
        activate = False
        y1 = y
        y2 = y - dy
        if y1 > y2:
            y1, y2 = y2, y1
        for n in range(36):
            string = gstring.strings[n]
            if string.y >= y1 and string.y <= y2 and x >= string.x and x <= 430 \
                    and not n + 1 in self.muted and self.strumming \
                    and string not in self.plectrum._husbands:
                string.play()
                activate = True
                if self.strumming:
                    self.plectrum.marry(string, activate)

    def update(self, dt):
        self.plectrum.update(dt)

    def draw(self):
        for b in button.Buttons.itervalues():
            b.draw()
