import pyglet

from colors import *
import button
import gstring
from plectrum import Plectrum
from scroller import Scroller
from highscore import Settings

class PointDisplayManager(object):
    def __init__(self):
        self.batch = pyglet.graphics.Batch()
        self.label_pool = []
        self.velocity = 90 
        for n in range(10):
            label = pyglet.text.Label('', font_size=16, color=Grey.rgba(), batch=self.batch)
            label.starting_y = 0
            self.label_pool.append(label)

    def add(self, text, x, y, color):
        for label in self.label_pool:
            if label.text == '':
                label.color = color.rgba()
                label.x = x
                label.y = y
                label.starting_y = y
                label.text = text
                break

    def update(self, dt):
        for label in self.label_pool:
            label.y -= dt * self.velocity
            if label.y < label.starting_y - 100:
                label.text = ''
                label.starting_y = 0

    def draw(self):
        self.batch.draw()

class Engine(object):
    def __init__(self, level):
        self.level = level
        self.scroller = Scroller()
        self.score = 0
        self.high_score = Settings.get_option(self.level)
        self.beat_interval = 0
        self.score_display = pyglet.text.Label('0', font_size = 60, x=100, y=450, color=Grey.rgba())
        self.high_display = pyglet.text.Label(str(self.high_score), font_size = 30, x=105, y=520, color=Grey.rgba(100))
        self.current_buttons = []
        self.velocity = 0
        self.beat = 1
        self.point_queue = 0
        self.point_display = PointDisplayManager()
        self.selected_beat = -1000
        self.next = 0
        self.saved = {}
        self.points = 0
        self.totalp = 0
        self.time = 0
        self.total = pyglet.text.Label('', font_size=30, x=93, y=410, color=Red.rgba())
        self.document = pyglet.text.document.FormattedDocument()
        self.layout = pyglet.text.layout.IncrementalTextLayout(self.document, 200, 300, True)
        self.layout.x = 460
        self.layout.y = 65
        self.beats_16 = []
        self.total_dt = 0
        self.load_parse_setup()

    def load_parse_setup(self):
        lines = file('data/levels/' + self.level, 'r').readlines()
        self.velocity = int(lines[0].strip())
        self.beat_interval = 60./self.velocity/16
        pixels = 60 / 16.

        self.document.text = lines[2][:-1]
        self.document.set_style(0, len(self.document.text), {'color': Grey.rgba()})
        self.display_text = True
        pyglet.clock.schedule_once(self.clear_text, 5)

        for line in lines[4:]:
            if line[0] in ['#', ' ', '\n', '\r']:
                continue
            values = line.split()
            measure = values[0].split('.')
            beat = (int(measure[0]) - 1) * 16 + int(measure[1]) + 1
            self.saved[beat] = (int(values[1]),  int(values[2]))
            self.totalp += 1
            self.scroller.spawn(int(values[1]), pixels * (beat - 1), self.velocity, beat)
            self.ending_time = beat / 16. + 4
        num = 460./self.velocity
        self.beat -= 460/self.velocity + 2
        self.beats_16 = self.saved.keys()
        self.beats_16.sort()
        pyglet.clock.schedule_once(self.tick, num - int(num))

    def clear_text(self, dt):
        self.display_text = False

    def tick(self, dt):
        self.beat += 1
        self.time = 0
        self.scroller.flash()
        pyglet.clock.schedule_once(self.tickplus, .075)

    def tickplus(self, dt):
        self.scroller.unflash()
            
    def on_strum(self, strings, buttons):
        win = False
        true_beat = self.beat * 16 + int(self.time/self.beat_interval)
        chord = []
        for beat in [true_beat - 2, true_beat - 1, true_beat, true_beat + 1, true_beat + 2]:
            try:
                chord.append((beat, self.saved[beat]))
            except KeyError:
                pass

        winnerc = None
        local_score = 0
        for button in buttons:
            for c in [c for c in chord if c[1]]:
                if button == c[1][0]:
                    region = ((strings[0] - 1) / 12) + 1
                    if region == c[1][1]:
                        # Got the right region
                        local_score += 75
                    winnerc = c[0]
                    if winnerc == true_beat:
                        # Got a perfect strum
                        local_score += 50
                    elif winnerc == true_beat - 1 or winnerc == true_beat + 1:
                        # Got a decent strum
                        local_score += 25
                    self.scroller.notes[winnerc].delete()
                    self.scroller.notes[winnerc] = None
                    self.saved[winnerc] = None
                    self.current_buttons = []
                    # Got the right chord
                    local_score += 100
                    self.points += 1
                    self.point_queue += local_score
                    self.point_display.add('+' + str(local_score), 535, 155, White)
                    break

    def select_strings(self, region):
        for n in range(36):
            gstring.strings[n].unselect()
            if (n / 12) + 1 == region:
                gstring.strings[n].select()

    def update(self, dt):
        if self.point_queue >= 5:
            self.point_queue -= 5
            self.score += 5
        self.time += dt
        period = 60./self.velocity
        if self.time > period:
            self.beat += 1
            self.time = self.time - period
            self.scroller.flash()
            pyglet.clock.schedule_once(self.tickplus, period + .075)

        if self.score > self.high_score:
            self.high_score = self.score
            self.high_display.text = str(self.high_score)
        true_beat = self.beat * 16 + int(self.time/self.beat_interval)
        if self.beats_16:
            info = self.saved[self.beats_16[0]]
            if info != None and self.beats_16[0] != self.selected_beat:
                self.select_strings(info[1])
                self.selected_beat = self.beats_16[0]
            if self.beats_16[0] < true_beat:
                self.beats_16.remove(self.beats_16[0])
        else:
            self.select_strings(-1)
        self.score_display.text = '%s' % self.score
        self.scroller.update(dt)
        self.point_display.update(dt)

    def draw(self):
        self.scroller.draw()
        self.score_display.draw()
        self.high_display.draw()
        if self.display_text:
            self.layout.draw()
        if self.beat > self.ending_time - 3:
            percent = float(self.points) / self.totalp * 100
            self.total.text = '%3s%%' %  int(percent)
            if percent == 100:
                self.total.color = Green.rgba()
            elif percent >= 75:
                self.total.color = Yellow.rgba()
            elif percent >= 50:
                self.total.color = Orange.rgba()
            self.total.draw()
        self.point_display.draw()

class PlayMode(object):
    def __init__(self, window, level):
        self.level = level
        self.engine = Engine(level)
        self.window = window
        self.muted = []
        self.paused = False
        self.plectrum = Plectrum()
        self.strumming = False
        self.buttons_pressed = []
        self.paused_text = pyglet.text.Label('* Paused *', font_size=48, x=400, y=300, anchor_x='center', anchor_y='center', color=White.rgba())
        self.paused_image = pyglet.resource.image('paused.png')

    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.ESCAPE:
            pyglet.clock.unschedule(self.engine.tick)
            pyglet.clock.unschedule(self.engine.tickplus)
            for string in gstring.strings:
                string.reset()
            self.window.select_mode()
        if symbol == pyglet.window.key.SPACE:
            self.paused = not self.paused
        try:
            b = button.Buttons[symbol]
            b.on = True
            self.buttons_pressed.append(button.button_list.index(b) + 1)
            for note in b.chord:
                self.muted.append(note)
                gstring.strings[note - 1].channel.stop()
            for n in self.muted:
                gstring.strings[n - 1].mute()
        except KeyError:
            pass

    def on_key_release(self, symbol, modifiers):
        try:
            b = button.Buttons[symbol]
            b.on = False
            self.buttons_pressed.remove(button.button_list.index(b) + 1)
            for note in b.chord:
                self.muted.remove(note)
            for n in range(36):
                if n + 1 not in self.muted:
                    gstring.strings[n].unmute()
        except KeyError:
            pass

    def on_mouse_press(self, x, y, buttons, modifiers):
        self.strumming = True

    def on_mouse_release(self, x, y, buttons, modifiers):
        self.strumming = False

    def on_mouse_drag(self, x, y, dx, dy, button, modifiers):
        activate = False
        y1 = y
        y2 = y - dy
        if y1 > y2:
            y1, y2 = y2, y1
        strings = []
        for n in range(36):
            string = gstring.strings[n]
            if string.y >= y1 and string.y <= y2 and x >= string.x and x <= 430 \
                    and not n + 1 in self.muted and self.strumming \
                    and string not in self.plectrum._husbands:
                string.channel.play(string.sound)
                strings.append(n + 1)
                activate = True
                if self.strumming:
                    self.plectrum.marry(string, activate)
        if strings and self.buttons_pressed:
            self.engine.on_strum(strings, self.buttons_pressed)

    def update(self, dt):
        if self.paused:
            return
        self.plectrum.update(dt)
        self.engine.update(dt)
        if self.engine.beat > self.engine.ending_time:
            for string in gstring.strings:
                string.reset()
            Settings.set_option(self.level, str(self.engine.high_score))
            Settings.save()
            self.window.select_mode()

    def draw(self):
        for b in button.Buttons.itervalues():
            b.draw()
        self.engine.draw()
        if self.paused:
            self.paused_image.blit(0,0)
            self.paused_text.draw()
