import os

import pyglet

from colors import *

class ItemSelect(object):
    def __init__(self, x, y, width, height, items):
        self.document = pyglet.text.document.FormattedDocument()
        line = 0
        for item in items:
            self.document.text += item + '\n'
        self.document.text = self.document.text[:-1]
        self.document.set_style(0, len(self.document.text), {'align': 'center', 'color': Grey.rgba(), 'font_size': 14})
        self.layout = pyglet.text.layout.IncrementalTextLayout(self.document, width, height, True)
        self.layout.x = x
        self.layout.y = y
        self.caret = pyglet.text.caret.Caret(self.layout)
        self.caret.position = 0
        self.caret.visible = False

    def draw(self):
        self.layout.draw()

class SelectMode(object):
    def __init__(self, window):
        self.window = window
        self.title = pyglet.resource.image('title.png')
        levels = os.listdir(os.path.join('data', 'levels'))
        for n in range(len(levels)):
            levels[n] = levels[n].replace('_', ' ')
        self.level_select = ItemSelect(460, 65, 200, 300, levels)

    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.ESCAPE:
            self.window.menu_mode()

    def on_key_release(self, symbol, modifiers):
        pass

    def on_mouse_press(self, x, y, buttons, modifiers):
        self.level_select.caret.on_mouse_press(x, y, buttons, modifiers)
        self.level_select.caret.visible = False
        start = self.level_select.document.get_paragraph_start(self.level_select.caret.position)
        end = self.level_select.document.get_paragraph_end(self.level_select.caret.position)
        level = self.level_select.document.text[start:end].strip()
        if level:
            self.window.play_mode(level.replace(' ', '_'))
                
    def on_mouse_scroll(self, x, y, scroll_x, scroll_y):
        self.level_select.caret.on_mouse_scroll(x, y, scroll_x, scroll_y)

    def on_mouse_release(self, x, y, buttons, modifiers):
        pass

    def on_mouse_drag(self, x, y, dx, dy, button, modifiers):
        pass

    def update(self, dt):
        pass

    def draw(self):
        self.title.blit(0, 0)
        self.level_select.draw()
